﻿import { Component, Inject, OnInit, OnDestroy } from "@angular/core";
import { Http } from "@angular/http";
import { ActivatedRoute } from "@angular/router";
import { TS } from "typescript-linq/TS";
import { deserialize } from "serializer.ts/Serializer";
import { Type } from "serializer.ts/Decorators";
import { EJAngular2Module } from "ej-angular2"; 
import { BrowserModule } from "@angular/platform-browser";
import * as $ from "jquery";

@Component({
    selector: "fetchdata",
    templateUrl: "./fetchdata.component.html"
})
export class FetchDataComponent implements OnInit, OnDestroy {
    public id: number;
    private sub: any;
    private originUrl: any;
    private http: Http;
    public project: Project;
    public filterType;
    public sections: ContractSection[];
    public detailTemp;
    public locale: string;
    public paging: boolean;
    public allowResize: boolean;
    invoices: Invoice[];

    constructor(http: Http, @Inject("ORIGIN_URL") originUrl: string, private route: ActivatedRoute) {
        this.http = http;
        this.originUrl = originUrl;
        this.filterType = { filterType: "excel" };
        this.locale = "de-DE";
        this.detailTemp = "#contractGridContents";
        this.paging = false;
        this.allowResize = false;
    }

    onOpen(e) {
        console.log("Triggers after the suggestion list is opened.");
    }


    loadProject() {
        this.http.get(this.originUrl + "/api/SampleData/Get", { params: { id: this.id } }).subscribe(result => {
            var loadedProject = deserialize<Project>(Project, result.json());
            this.sections = loadedProject.contractSections;
            this.paging = (this.sections.length > 8);
            this.allowResize = (this.sections.length > 0);
            if (this.sections.length === 0) return;
            for (var section of this.sections) {
                section.calculateSumOfTargets();
            }
            this.project = loadedProject;
        });

    }

    ngOnInit() {
        this.sub = this.route.params.subscribe(params => {
            this.id = +params["id"]; // (+) converts string 'id' to a number

            this.loadProject();

        });
    }

    ngOnDestroy() {
        this.sub.unsubscribe();
    }

    onDetailsDataBound(e: any) {

        // Here you can get the parent details from "data". EmployeeID is the unique column value in parent row.
        var selectedId = parseInt(e.data["id"]);
        const enumerator = new TS.Linq.Enumerator<ContractSection>(this.sections);
        const data = enumerator.where(c => c.id === selectedId).first().contracts;
        if (data.length === 0) {
            return;
        }

        e.detailsElement.find("#contractGrid").ejGrid({
            allowPaging: data.length > 5,
            allowReordering: true,
            pageSettings: { pageSize: 5 },
            allowSorting: true,
            allowResizeToFit: data.length > 0,
            allowFiltering: data.length > 0,
            filterSettings: this.filterType,
            locale: this.locale,
            dataSource: data,
            detailsTemplate: "#invoiceGridContents",
            dataBound: this.onContractsDataBound,
            rowDataBound: this.onContractsDataBound,
            detailsDataBound: this.onInvoiceDetailsDataBound,
            detailsExpand: this.onExpandDetails,
            recordDoubleClick: this.onContractRecordDoubleClick,
            showStackedHeader: true,
            stackedHeaderRows: [{
                stackedHeaderColumns: [{
                    headerText: "Aufträge",
                    column: "number,targetBudget,currentBudget,budgetConsumption,sumOfWarrentiesOrSecurities,creditorNumber,creditor.name"
                }]
            }],
            columns: [
                { field: "number", headerText: "Nummer", cssClass: "{{:childCss}}" },
                { field: "targetBudget", headerText: "Soll", format: "{0:N2}", textAlign: "Right"},
                { field: "currentBudget", headerText: "Soll aus Aufrägen", format: "{0:N2}", textAlign: "Right"},
                { field: "budgetConsumption", headerText: "Auftragsauslastung", template: true, templateID: "#columnTemp", textAlign: "Center"},
                { field: "sumOfWarrentiesOrSecurities", headerText: "Bürgsachft/Sicherheit", format: "{0:N2}", textAlign: "Right"},
                { field: "creditorNumber", headerText: "Kreditor" },
                { field: "creditor.name", headerText: "Firma" }
            ]
        });
    }
    
    onExpandDetails(args: any) {
        this.invoices = args.masterData.invoices;
    };

    onContractsDataBound(e: any) {
        $("#contractGrid").find("tr > .noChild").closest("tr").find(".e-icon").remove();
        $("#contractGrid").find("tr > .noChild").closest("tr").find(".e-detailrowcollapse").unbind("click");
        $("#contractGrid").find("tr > .noChild").closest("tr").find(".e-detailrowcollapse")
            .removeClass("e-detailrowcollapse").addClass("e-rowcell");
    }

    onInvoiceDetailsDataBound(e: any) {
        if (this.invoices.length === 0) {
            return;
        };

        e.detailsElement.find("#invoiceGrid").ejGrid({
            allowPaging: this.invoices.length > 5,
            allowReordering: true,
            pageSettings: { pageSize: 5 },
            allowSorting: true,
            allowResizeToFit: this.invoices.length > 0,
            allowFiltering: this.invoices.length > 0,
            filterSettings: { filterType: "excel" },
            locale: "de-DE",
            dataSource: this.invoices,
            showStackedHeader: true,
            stackedHeaderRows: [
                {
                    stackedHeaderColumns: [
                        {
                            headerText: "Rechnungen",
                            column: "number,currentNumber,amount,sumDeductions,approvedAmount,isApproved"
                        }
                    ]
                }
            ],
            columns: [
                { field: "number", headerText: "Rechnungsnummer" },
                { field: "currentNumber", headerText: "lfd. #", textAlign: "Right" },
                { field: "amount", headerText: "Betrag", format: "{0:N2}", textAlign: "Right" },
                { field: "sumDeductions", headerText: "Summe Abzüge", format: "{0:N2}", textAlign: "Right" },
                { field: "approvedAmount", headerText: "Auszuweisender Betrag", format: "{0:N2}", textAlign: "Right" },
                { field: "isApproved", headerText: "Geprüft", textAlign: "Center" }
            ],
            recordDoubleClick: this.onInvoiceRecordDoubleClick
        });
        //.on("recordClick", function () { alert("Test"); });
        // $("#invoiceGrid").find(".e-row").dblclick(function() { alert("bla"); }); //this.onInvoiceRecordDoubleClick)};
    };

    onContractSectionRecordDoubleClick(e: any) {
        alert(e.rowData.id);
    };
    onContractRecordDoubleClick(e: any) {
        alert(e.rowData.id);
    };
    onInvoiceRecordDoubleClick(e: any) {
        alert("Test");
    };
}

class BudgetBase {
    id: number;
    currentBudget: number;
    targetBudget: number;
    hasExceededBudget: boolean;
    budgetConsumption: number;
    budgetConsumptionPercentage: number;
    //get budgetConsumption(): number {
    //    if (this.targetBudget === 0 || this.currentBudget === 0) return 0;
    //    return (this.currentBudget / this.targetBudget);
    //}
    //set budgetConsumption(x: number) { }

    //get budgetConsumptionPercentage(): number { return this.budgetConsumption * 100; }
    //set budgetConsumptionPercentage(x: number) { }
    

}

class ContractSection extends BudgetBase {
    projectId: number;
    costTypeId: number;
    costTypeDescription: string;
    constructionTime: number;
    @Type(() => Contract)
    contracts: Contract[];
    sumOfContractTargets: number;

    public calculateSumOfTargets(): void {
        if (this.contracts.length === 0) return;
        const enumerator = new TS.Linq.Enumerator<Contract>(this.contracts);
        this.sumOfContractTargets = enumerator.select(c => c.targetBudget).sum();
    }

    get targetConsumption(): number {
        if (this.targetBudget === 0) return 6.66;
        if (this.sumOfContractTargets === null) return 0;
        return (this.sumOfContractTargets / this.targetBudget);
    }
    set targetConsumption(x: number) { }

}

class Payment extends BudgetBase {
    discount: number;
    tradeDiscount: number;
    apportionment: number;
    apportionmentValue: number;
    warrentyRetentionByBond: number;
    warrentyRetentionByBondValue: number;
    riskInsurance: number;
    riskInsuranceValue: number;
    payoutPercentage: number;
    payoutPercentageValue: number;
    payoutPercentageSafekeeping: number;
    payoutPercentageSafekeepingValue: number;
    paymentGuarantee: number;
    documentFilepath: string;
    comment: string;
    sumOfWarrentiesOrSecurities: number;
}

class Contract extends Payment {
    mainContractNumber: string;
    creditorNumber: string;
    costTypeDescription: string;
    contractSectionId: number;
    number: string;
    sumOfWarrentiesOrSecurities: number;
    @Type(() => Invoice)
    invoices: Invoice[];
    get childCss(): string {
        if (this.invoices.length === 0)
            return "noChild"
        else return "child";
    }
}

class Invoice extends Payment {
    id: number;
    contractId: number;
    creditorNumber: string;
    contractNumber: string;
    number: string;
    currentNumber: number;
    isApproved: boolean;
    isPartial: boolean;
    isLast: boolean;
    isBond: boolean;
    amount: number;
    approvedAmount: number;
    comment: string;
    vat: number;
    otherPayoutPercentage: number;
    otherPayoutPercentageValue: number;
    constructionDebris: number;
    defectRetention: number;
    date: Date;
    reviewDate: Date;
    reviewedAmount: number;
    sumOfPreviousApprovedAmount: number;
    sumDeductions: number;
    sumOfWarrentiesOrSecurities;
}

class Project extends BudgetBase {
    id: number;
    number: string;
    description: string;
    isActive: boolean;
    @Type(() => ContractSection)
    contractSections: ContractSection[];
}
